# Copyright (C) 2006 by Aiwota Programmer
# aiwotaprog@tetteke.tk
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from HTMLParser import HTMLParser
import htmlentitydefs


class BareHTMLParser(HTMLParser):
    """Parses html by the minimal necessity

    to_out_func format is:
    def some_func(untied_data, is_bold, href):
    where untied_data is non markuped string
    and is_bold is whether untied_data is bold or not
    and href is url anchor if exists
    """

    def __init__(self, to_out_func):
        HTMLParser.__init__(self)
        self.to_out_func = to_out_func
        self.bold = False
        self.href = None

    def reset_func(self, to_out_func):
        self.to_out_func = to_out_func

    def to_out(self, data):
        self.to_out_func(data, self.bold, self.href)

    # handle_* are overriden methods

    def handle_starttag(self, tag, attr):
        if tag == "b":
            self.bold = True
        elif tag == "br":
            self.to_out("\n")
        elif tag == "a":
            for item in attr:
                if item[0] == "href":
                    self.href = item[1]

    def handle_endtag(self, tag):
        if tag == "b":
            self.bold = False
        elif tag == "a":
            self.href = None

    def handle_data(self, data):
        self.to_out(data)

    def handle_charref(self, ref):
        data = None
        try:
            data = unichr(int(ref))
        except:
            data = "&#"+ref+";"
        self.to_out(data)

    def handle_entityref(self, name):
        if name in htmlentitydefs.name2codepoint:
            codepoint = htmlentitydefs.name2codepoint[name]
            self.to_out(unichr(codepoint))
        else:
            self.to_out("&"+name+";")
