# Copyright (C) 2006 by Aiwota Programmer
# aiwotaprog@tetteke.tk
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import os.path
import codecs
import re
import pango
import urllib2

import misc
import datfile
import barehtmlparser
import idxfile
import session
import board_window

GLADE_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                         "..", "data")
GLADE_FILENAME = "thread_window.glade"

def open_thread(uri):
    if not uri:
        raise ValueError, "parameter must not be empty"

    winwrap = session.get_window(uri)
    if winwrap:
        # already opened
        winwrap.window.present()
        pass
    else:
        win_wrap = WinWrap(uri)
        session.window_created(uri, win_wrap)


class WinWrap:

    def __init__(self, uri):
        from BbsType import bbs_type_judge_uri
        from BbsType import bbs_type_exception
        self.bbs_type = bbs_type_judge_uri.get_type(uri)
        if not self.bbs_type.is_thread():
            raise bbs_type_exception.BbsTypeError, \
                  "the uri does not represent thread: " + uri
        self.bbs = self.bbs_type.bbs_type
        self.board = self.bbs_type.board
        self.thread = self.bbs_type.thread
        self.host = self.bbs_type.host
        self.uri = self.bbs_type.uri
        self.size = 0
        self.num = 0
        self.title = None

        glade_path = os.path.join(GLADE_DIR, GLADE_FILENAME)
        self.widget_tree = gtk.glade.XML(glade_path)
        self.window = self.widget_tree.get_widget("thread_window")
        self.textview = self.widget_tree.get_widget("textview")
        self.textbuffer = self.textview.get_buffer()
        self.enditer = self.textbuffer.get_end_iter()
        self.boldtag = self.textbuffer.create_tag(weight=pango.WEIGHT_BOLD)
        self.leftmargintag = self.textbuffer.create_tag()
        self.leftmargintag.set_property("left-margin", 20)

        sigdic = {"on_toolbutton_refresh_clicked": self.update,
                  "on_refresh_activate": self.update,
                  "on_close_activate": self.on_close_activate,
                  "on_quit_activate": self.on_quit_activate,
                  "on_show_board_activate": self.on_show_board_activate,
                  "on_thread_window_destroy": self.on_thread_window_destroy}
        self.widget_tree.signal_autoconnect(sigdic)

        self.load_dat()

    def on_close_activate(self, widget):
        self.window.destroy()

    def on_thread_window_destroy(self, widget):
        -1

    def on_quit_activate(self, widget):
        session.main_quit()

    def on_show_board_activate(self, widget):
        board_window.open_board(self.bbs_type.get_uri_base())

    def http_get_dat(self, on_get_res):
        datfile_url = self.bbs_type.get_dat_uri()

        idx_dic = idxfile.load_idx(self.bbs, self.board, self.thread)
        lastmod = idx_dic["lastModified"]
        etag = idx_dic["etag"]

        req = urllib2.Request(datfile_url)
        if self.size > 0:
            req.add_header("Range", "bytes=" + str(self.size) + "-")
        if lastmod:
            req.add_header("If-Modified-Since", lastmod)
        if etag:
            req.add_header("If-None-Match", etag)
        print req.headers

        res = urllib2.urlopen(req)
        headers = res.info()
        print headers

        line = res.readline()
        maybe_incomplete = False
        while line:
            if not line.endswith("\n"):
                maybe_incomplete = True
                print "does not end with \\n. maybe incomplete"
                break
            on_get_res(line)
            line = res.readline()

        res.close()

        if maybe_incomplete:
            lastmod = None
            etag = None
        else:
            if "Last-Modified" in headers:
                lastmod = headers["Last-Modified"]
            if "ETag" in headers:
                etag = headers["Etag"]

        if self.num > 0:
            if not self.title:
                self.title = datfile.get_title_from_dat(
                    self.bbs, self.board, self.thread)
                if self.title:
                    self.window.set_title(self.title)
            # save idx
            idx_dic = {"title": self.title, "lineCount": self.num,
                   "lastModified": lastmod, "etag": etag}
            idxfile.save_idx(self.bbs, self.board, self.thread, idx_dic)

            session.thread_idx_updated(
                self.bbs_type.get_thread_uri(), idx_dic)

    def update(self, widget=None):
        line_count = datfile.get_dat_line_count(
            self.bbs, self.board, self.thread)

        if line_count > self.num:
            datfile.load_dat_partly(
                self.bbs, self.board, self.thread,
                self.append_rawres_to_buffer, self.num+1)

        dat_path = misc.get_thread_dat_path(self.bbs, self.board, self.thread)
        dat_file = file(dat_path, "a+")

        def save_line_and_append_to_buffer(line):
            dat_file.seek(self.size)
            dat_file.write(line)
            self.append_rawres_to_buffer(line)

        self.http_get_dat(save_line_and_append_to_buffer)
        dat_file.close()

    def load_dat(self):
        self.size = 0
        self.num = 0

        self.title = datfile.get_title_from_dat(
            self.bbs, self.board, self.thread)
        if self.title:
            self.window.set_title(self.title)

        datfile.load_dat(self.bbs, self.board, self.thread,
                         self.append_rawres_to_buffer)
        self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)


    def append_rawres_to_buffer(self, line):
        self.size += len(line)
        self.num += 1

        h = lambda name,mail,date,msg: self.reselems_to_buffer(
            self.num, name, mail, date, msg)

        datfile.split_line_to_elems(line.decode("cp932", "replace"), h)
        
    def reselems_to_buffer(self, num, name, mail, date, msg):
        # number
        self.textbuffer.insert(self.enditer, str(num) + " ")

        # name
        p = barehtmlparser.BareHTMLParser(self.untiedata_to_buffer)
        p.feed("<b>" + name + "</b>")
        p.close()

        # mail
        self.textbuffer.insert(self.enditer, "[" + mail + "]")

        # date
        p.feed(date)
        p.close()
        self.textbuffer.insert(self.enditer, "\n")

        # msg
        p.reset_func(self.untiedata_to_buffer_with_leftmargin)
        p.feed(msg)
        p.close()

        self.textbuffer.insert(self.enditer, "\n\n")

    def untiedata_to_buffer(self, data, bold, href):
        if bold:
            self.textbuffer.insert_with_tags(self.enditer, data, self.boldtag)
        else:
            self.textbuffer.insert(self.enditer, data)

    def untiedata_to_buffer_with_leftmargin(self, data, bold, href):
        if bold:
            self.textbuffer.insert_with_tags(self.enditer, data,
                                             self.boldtag, self.leftmargintag)
        else:
            self.textbuffer.insert_with_tags(self.enditer, data,
                                             self.leftmargintag)
